@@ -1,15 +1,16 @@
 use std::fmt::{Display, Formatter};
+use std::io;
+use std::io::Write;
 use std::path::Path;
-use std::process::Command;
 
-use color_eyre::eyre::eyre;
 use color_eyre::eyre::Context;
 use color_eyre::eyre::Result;
+use color_eyre::eyre::{eyre, OptionExt};
 use tracing::{debug, error, warn};
 use wildmatch::WildMatch;
 
 use crate::command::CommandExt;
-use crate::error::{self, TopgradeError};
+use crate::error::{SkipStep, TopgradeError};
 use crate::terminal::print_separator;
 use crate::{execution_context::ExecutionContext, utils::require};
 use rust_i18n::t;
@@ -21,6 +22,9 @@ use rust_i18n::t;
 // themselves or when using docker-compose.
 const NONEXISTENT_REPO: &str = "repository does not exist";
 
+// A string found in the output of docker when Docker Desktop is not running.
+const DOCKER_NOT_RUNNING: &str = "We recommend to activate the WSL integration in Docker Desktop settings.";
+
 /// Uniquely identifies a `Container`.
 #[derive(Debug)]
 struct Container {
@@ -60,8 +64,8 @@ impl Display for Container {
 /// "REGISTRY/[PATH/]CONTAINER_NAME:TAG"
 ///
 /// Containers specified in `ignored_containers` will be filtered out.
-fn list_containers(crt: &Path, ignored_containers: Option<&Vec<String>>) -> Result<Vec<Container>> {
-    let ignored_containers = ignored_containers.map(|patterns| {
+fn list_containers(ctx: &ExecutionContext, crt: &Path) -> Result<Vec<Container>> {
+    let ignored_containers = ctx.config().containers_ignored_tags().map(|patterns| {
         patterns
             .iter()
             .map(|pattern| WildMatch::new(pattern))
@@ -72,9 +76,23 @@ fn list_containers(crt: &Path, ignored_containers: Option<&Vec<String>>) -> Resu
         "Querying '{} image ls --format \"{{{{.Repository}}}}:{{{{.Tag}}}}/{{{{.ID}}}}\"' for containers",
         crt.display()
     );
-    let output = Command::new(crt)
-        .args(["image", "ls", "--format", "{{.Repository}}:{{.Tag}} {{.ID}}"])
-        .output_checked_with_utf8(|_| Ok(()))?;
+
+    let sudo = if ctx.config().containers_use_sudo() {
+        Some(ctx.require_sudo()?)
+    } else {
+        None
+    };
+
+    // TODO: This should run even when dry-running. Blocked by #1227.
+    let output = if let Some(sudo) = sudo {
+        sudo.execute(ctx, crt)?
+            .args(["image", "ls", "--format", "{{.Repository}}:{{.Tag}} {{.ID}}"])
+            .output_checked_utf8()?
+    } else {
+        ctx.execute(crt)
+            .args(["image", "ls", "--format", "{{.Repository}}:{{.Tag}} {{.ID}}"])
+            .output_checked_utf8()?
+    };
 
     let mut retval = vec![];
     for line in output.stdout.lines() {
@@ -99,7 +117,12 @@ fn list_containers(crt: &Path, ignored_containers: Option<&Vec<String>>) -> Resu
 
         // line is of format: `Repository:Tag ImageID`, e.g., `nixos/nix:latest d80fea9c32b4`
         let split_res = line.split(' ').collect::<Vec<&str>>();
-        assert_eq!(split_res.len(), 2);
+        if split_res.len() != 2 {
+            return Err(eyre!(format!(
+                "Got erroneous output from `{} image ls --format \"{{.Repository}}:{{.Tag}} {{.ID}}\"; Expected line to split into 2 parts",
+                crt.display()
+            )));
+        }
         let (repo_tag, image_id) = (split_res[0], split_res[1]);
 
         if let Some(ref ignored_containers) = ignored_containers {
@@ -114,13 +137,25 @@ fn list_containers(crt: &Path, ignored_containers: Option<&Vec<String>>) -> Resu
             crt.display(),
             image_id
         );
-        let inspect_output = Command::new(crt)
-            .args(["image", "inspect", image_id, "--format", "{{.Os}}/{{.Architecture}}"])
-            .output_checked_with_utf8(|_| Ok(()))?;
+        // TODO: This should run even when dry-running. Blocked by #1227.
+        let inspect_output = if let Some(sudo) = sudo {
+            sudo.execute(ctx, crt)?
+                .args(["image", "inspect", image_id, "--format", "{{.Os}}/{{.Architecture}}"])
+                .output_checked_utf8()?
+        } else {
+            ctx.execute(crt)
+                .args(["image", "inspect", image_id, "--format", "{{.Os}}/{{.Architecture}}"])
+                .output_checked_utf8()?
+        };
         let mut platform = inspect_output.stdout;
         // truncate the tailing new line character
         platform.truncate(platform.len() - 1);
-        assert!(platform.contains('/'));
+        if !platform.contains('/') {
+            return Err(eyre!(format!(
+                "Got erroneous output from `{} image ls --format \"{{.Repository}}:{{.Tag}} {{.ID}}\"; Expected platform to contain '/'",
+                crt.display()
+            )));
+        }
 
         retval.push(Container::new(repo_tag.to_string(), platform));
     }
@@ -131,24 +166,70 @@ fn list_containers(crt: &Path, ignored_containers: Option<&Vec<String>>) -> Resu
 pub fn run_containers(ctx: &ExecutionContext) -> Result<()> {
     // Check what runtime is specified in the config
     let container_runtime = ctx.config().containers_runtime().to_string();
+    let use_sudo = ctx.config().containers_use_sudo();
     let crt = require(container_runtime)?;
     debug!("Using container runtime '{}'", crt.display());
 
     print_separator(t!("Containers"));
-    let mut success = true;
-    let containers =
-        list_containers(&crt, ctx.config().containers_ignored_tags()).context("Failed to list Docker containers")?;
+
+    let sudo = if use_sudo { Some(ctx.require_sudo()?) } else { None };
+
+    // TODO: This should run even when dry-running. Blocked by #1227.
+    let output = if let Some(sudo) = sudo {
+        sudo.execute(ctx, &crt)?.arg("--help").output_checked_with(|_| Ok(()))?
+    } else {
+        ctx.execute(&crt).arg("--help").output_checked_with(|_| Ok(()))?
+    };
+
+    let status_code = output
+        .status
+        .code()
+        .ok_or_eyre("Couldn't get status code (terminated by signal)")?;
+    let stdout = std::str::from_utf8(&output.stdout).wrap_err("Expected output to be valid UTF-8")?;
+    if stdout.contains(DOCKER_NOT_RUNNING) && status_code == 1 {
+        // Write the output
+        io::stdout().write_all(&output.stdout)?;
+        io::stderr().write_all(&output.stderr)?;
+        // Don't crash, but don't be silent either.
+        // This can happen in other ways than Docker Desktop not running, but even in those cases
+        //  we don't want to crash, since the containers step is enabled by default.
+        warn!(
+            "{} seems to be non-functional right now (see above). Is WSL integration enabled for Docker Desktop? Is Docker Desktop running?",
+            crt.display()
+        );
+        return Err(SkipStep(format!(
+            "{} seems to be non-functional right now. Possibly WSL integration is not enabled for Docker Desktop, or Docker Desktop is not running.",
+            crt.display()
+        )).into());
+    } else if !output.status.success() {
+        // Write the output
+        io::stdout().write_all(&output.stdout)?;
+        io::stderr().write_all(&output.stderr)?;
+        // If we saw the message, but the code is not 1 (e.g. 0, or a non-1 failure), crash, as we expect a 1.
+        // If we did not see the message, it's broken in some way we do not understand.
+        return Err(eyre!(
+            "{0} seems to be non-functional (`{0} --help` returned non-zero exit code {1})",
+            crt.display(),
+            status_code,
+        ));
+    }
+
+    let containers = list_containers(ctx, &crt).context("Failed to list Docker containers")?;
     debug!("Containers to inspect: {:?}", containers);
 
-    for container in containers.iter() {
+    for container in &containers {
         debug!("Pulling container '{}'", container);
-        let args = vec![
-            "pull",
-            container.repo_tag.as_str(),
-            "--platform",
-            container.platform.as_str(),
-        ];
-        let mut exec = ctx.run_type().execute(&crt);
+        let mut args = vec!["pull", container.repo_tag.as_str()];
+        if container.platform.as_str() != "/" {
+            args.push("--platform");
+            args.push(container.platform.as_str());
+        }
+
+        let mut exec = if let Some(sudo) = sudo {
+            sudo.execute(ctx, &crt)?
+        } else {
+            ctx.execute(&crt)
+        };
 
         if let Err(e) = exec.args(&args).status_checked() {
             error!("Pulling container '{}' failed: {}", container, e);
@@ -170,27 +251,33 @@ pub fn run_containers(ctx: &ExecutionContext) -> Result<()> {
                 continue;
             }
 
-            success = false;
+            return Err(e);
         }
     }
 
-    if ctx.config().cleanup() {
+    if ctx.config().containers_system_prune() {
+        // Run system prune to clean up unused containers, networks, and build cache
+        if let Some(sudo) = sudo {
+            sudo.execute(ctx, &crt)?
+                .args(["system", "prune", "--force"])
+                .status_checked()?
+        } else {
+            ctx.execute(&crt)
+                .args(["system", "prune", "--force"])
+                .status_checked()?
+        }
+    // Only run `image prune` if we don't run `system prune`
+    } else if ctx.config().cleanup() {
         // Remove dangling images
         debug!("Removing dangling images");
-        if let Err(e) = ctx
-            .run_type()
-            .execute(&crt)
-            .args(["image", "prune", "-f"])
-            .status_checked()
-        {
-            error!("Removing dangling images failed: {}", e);
-            success = false;
+        if let Some(sudo) = sudo {
+            sudo.execute(ctx, &crt)?
+                .args(["image", "prune", "-f"])
+                .status_checked()?
+        } else {
+            ctx.execute(&crt).args(["image", "prune", "-f"]).status_checked()?
         }
     }
 
-    if success {
-        Ok(())
-    } else {
-        Err(eyre!(error::StepFailed))
-    }
+    Ok(())
 }
